local M = {}

local loadedSounds = {}

local engineSound = nil
local windSound = nil
local cameraNode = 0
local pitch_e = nil	

local function nop()
end

local function playSoundOnceAtNode(soundName, nodeID, volume)
    local snd = gameEngine:playSFXOnce(soundName)
    snd:setVolume(volume)
    obj:attachSFXNode(snd, nodeID)
end

local wheelsSounds = nil

local function playWheelSound(wheelID, soundName, vol, pitch)
    ws = wheelsSounds[wheelID][soundName]
    if vol < 0.1 then vol = 0 end
    if vol == 0 then 
        pitch = 0
    else
        if math.abs(vol - ws.lastVol) / vol < 0.05 and math.abs(pitch - ws.lastPitch) < 0.05 then return end
    end
    if vol == ws.lastvol then return end
    ws.lastVol = vol
    ws.lastPitch = pitch
    ws.obj:setVolume(vol)
    ws.obj:setPitch(pitch)
end

local function update(dt)
   if v.data.engine ~= nil and v.data.engine.maxRPM ~= nil then
		en = pitch_e:get(thrusters.total_energy/4000)
        obj:setPitch(engineSound, math.min(50,en+0.5))
		if thrusters.global_enable then
			volume = en+0.7
		else 
			volume = 0 
		end
        obj:setVolume(engineSound, volume)
    end
    
    -- wind

    local speed = obj:getAirSpeed() -- speed against wind
    local vol = (speed * speed * 0.001)
    local pitch = speed / 60
    if vol > 1 then vol = 1 end
    windSound:setVolume(vol*0.8)
    windSound:setPitch(pitch)
	
	
end

local function addWheelSound(wheelID, wd, soundName)
    if wheelsSounds == nil then wheelsSounds = {} end
    if wheelsSounds[wheelID] == nil then wheelsSounds[wheelID] = {} end
    
    local s = gameEngine:createSFXSource(soundName)
    if s == nil then 
        M.update = nop
        M.playSoundOnceAtNode = nop
        return nil 
    end
    s:setVolume(0)
    s:setPitch(1)
    s:play(-1)
    obj:attachSFXNode(s, wd.node1)
    wheelsSounds[wheelID][soundName] = {obj = s, lastVol = 0, lastPitch = 0}
    table.insert(loadedSounds, s)
end

local function createSFXSource(SFXname)
    local var = gameEngine:createSFXSource(SFXname)
    if var == nil then 
        M.update = nop
        M.playSoundOnceAtNode = nop
        return nil 
    end
    var:setVolume(0)
    var:play(-1)
    obj:attachSFXNode(var, cameraNode)
    table.insert(loadedSounds, var)
    return var
end

local function init()
	pitch_e = newExponentialSmoothing(10)
	
    if v.data.camerasInternal ~= nil then
        local k, v = next(v.data.camerasInternal)
        cameraNode = v.camNodeID
    end

    if engineSound == nil then
        engineSound = createSFXSource("hover_en")
    end

    if windSound == nil then
        windSound = createSFXSource("WindTestSound")
    end    

    if wheelsSounds == nil then
        for wi,wd in pairs(drivetrain.wheels) do
            addWheelSound(wi, wd, "RollingTestSound")
            addWheelSound(wi, wd, "SkidTestSound")
        end
    end
end

local function destroy()
    for k,v in pairs(loadedSounds) do
        if v and v:isValid() then
            gameEngine:deleteSFXSource(v, true)
            v:destroy()
        end
    end
    loadedSounds = {}
    wheelsSounds = nil
    engineSound = nil
    windSound = nil
end

M.destroy = destroy
M.init = init
M.update = update
return M